/*
 * Copyright (c) 2006-2021, RT-Thread Development Team
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Change Logs:
 * Date           Author       Notes
 * 2021-10-31     Administrator       the first version
 */
#include "lcd/lcd_init.h"
#include "lcd/lcd_font.h"
#include <dfs_posix.h>
#include "rtthread.h"

void LCD_Fill(uint16_t xsta,uint16_t ysta,uint16_t xend,uint16_t yend,uint16_t color)
{
    uint16_t i,j;
    LCD_Address_Set(xsta,ysta,xend-1,yend-1);
    for(i=ysta;i<yend;i++)
    {
        for(j=xsta;j<xend;j++)
        {
            LCD_WR_DATA(color);
        }
    }
}

void LCD_DrawPoint(uint16_t x,uint16_t y,uint16_t color)
{
    LCD_Address_Set(x,y,x,y);
    LCD_WR_DATA(color);
}

void LCD_DrawLine(uint16_t x1,uint16_t y1,uint16_t x2,uint16_t y2,uint16_t color)
{
    uint16_t t;
    int xerr=0,yerr=0,delta_x,delta_y,distance;
    int incx,incy,uRow,uCol;
    delta_x=x2-x1;
    delta_y=y2-y1;
    uRow=x1;
    uCol=y1;
    if(delta_x>0)incx=1;
    else if (delta_x==0)incx=0;
    else {incx=-1;delta_x=-delta_x;}
    if(delta_y>0)incy=1;
    else if (delta_y==0)incy=0;
    else {incy=-1;delta_y=-delta_y;}
    if(delta_x>delta_y)distance=delta_x;
    else distance=delta_y;
    for(t=0;t<distance+1;t++)
    {
        LCD_DrawPoint(uRow,uCol,color);
        xerr+=delta_x;
        yerr+=delta_y;
        if(xerr>distance)
        {
            xerr-=distance;
            uRow+=incx;
        }
        if(yerr>distance)
        {
            yerr-=distance;
            uCol+=incy;
        }
    }
}

void LCD_DrawRectangle(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2,uint16_t color)
{
    LCD_DrawLine(x1,y1,x2,y1,color);
    LCD_DrawLine(x1,y1,x1,y2,color);
    LCD_DrawLine(x1,y2,x2,y2,color);
    LCD_DrawLine(x2,y1,x2,y2,color);
}


void Draw_Circle(uint16_t x0,uint16_t y0,uint8_t r,uint16_t color)
{
    int a,b;
    a=0;b=r;
    while(a<=b)
    {
        LCD_DrawPoint(x0-b,y0-a,color);             //3
        LCD_DrawPoint(x0+b,y0-a,color);             //0
        LCD_DrawPoint(x0-a,y0+b,color);             //1
        LCD_DrawPoint(x0-a,y0-b,color);             //2
        LCD_DrawPoint(x0+b,y0+a,color);             //4
        LCD_DrawPoint(x0+a,y0-b,color);             //5
        LCD_DrawPoint(x0+a,y0+b,color);             //6
        LCD_DrawPoint(x0-b,y0+a,color);             //7
        a++;
        if((a*a+b*b)>(r*r))
        {
            b--;
        }
    }
}


void LCD_ShowChar(uint16_t x,uint16_t y,uint8_t num,uint16_t fc,uint16_t bc,uint8_t sizey,uint8_t mode)
{
    uint8_t temp,sizex,t,m=0;
    uint16_t i,TypefaceNum;
    uint16_t x0=x;
    sizex=sizey/2;
    TypefaceNum=(sizex/8+((sizex%8)?1:0))*sizey;
    num=num-' ';
    LCD_Address_Set(x,y,x+sizex-1,y+sizey-1);
    for(i=0;i<TypefaceNum;i++)
    {
        if(sizey==12)temp=ascii_1206[num][i];
        else if(sizey==16)temp=ascii_1608[num][i];
        else if(sizey==24)temp=ascii_2412[num][i];
        else if(sizey==32)temp=ascii_3216[num][i];
        else return;
        for(t=0;t<8;t++)
        {
            if(!mode)
            {
                if(temp&(0x01<<t))LCD_WR_DATA(fc);
                else LCD_WR_DATA(bc);
                m++;
                if(m%sizex==0)
                {
                    m=0;
                    break;
                }
            }
            else//鍙犲姞妯″紡
            {
                if(temp&(0x01<<t))LCD_DrawPoint(x,y,fc);//鐢讳竴涓偣
                x++;
                if((x-x0)==sizex)
                {
                    x=x0;
                    y++;
                    break;
                }
            }
        }
    }
}



void LCD_ShowString(uint16_t x,uint16_t y,const uint8_t *p,uint16_t fc,uint16_t bc,uint8_t sizey,uint8_t mode)
{
    while(*p!='\0')
    {
        LCD_ShowChar(x,y,*p,fc,bc,sizey,mode);
        x+=sizey/2;
        p++;
    }
}


/******************************************************************************
      鍑芥暟璇存槑锛氭樉绀烘暟瀛�
      鍏ュ彛鏁版嵁锛歮搴曟暟锛宯鎸囨暟
      杩斿洖鍊硷細  鏃�
******************************************************************************/
uint32_t mypow(uint8_t m,uint8_t n)
{
    uint32_t result=1;
    while(n--)result*=m;
    return result;
}



void LCD_ShowIntNum(uint16_t x,uint16_t y,uint16_t num,uint8_t len,uint16_t fc,uint16_t bc,uint8_t sizey)
{
    uint8_t t,temp;
    uint8_t enshow=0;
    uint8_t sizex=sizey/2;
    for(t=0;t<len;t++)
    {
        temp=(num/mypow(10,len-t-1))%10;
        if(enshow==0&&t<(len-1))
        {
            if(temp==0)
            {
                LCD_ShowChar(x+t*sizex,y,' ',fc,bc,sizey,0);
                continue;
            }else enshow=1;

        }
        LCD_ShowChar(x+t*sizex,y,temp+48,fc,bc,sizey,0);
    }
}

static uint8_t Find_Chinese(uint8_t* p, uint8_t* buffer)
{
    int fd;
    uint8_t High8bit,Low8bit;
    uint8_t error[]="Please put the sys folder which include the font file HZK16.bin into the SD  card  root directory. ";
    High8bit=*p;
    Low8bit=*(p+1);
    //f_mount(0, &fs);
    //rt_kprintf("High8bit = %2x,Low8bit = %2x\r\n",High8bit,Low8bit);
    fd = open("/GBK16X16.bin", O_WRONLY);/*ֿļGBK16X16.bin*/
    if (fd < 0)
    {
        //LCD_ShowString(32,100,error,RED ,BLACK );
        rt_kprintf("fd < 0\r\n");
        rt_kprintf("fd = %d\r\n",fd);
        return 1;
    }
    else {

        //rt_kprintf("open CnFont16.bin success\r\n");
    }

    //lseek(fd, 32*((High8bit-0xa0-1)*94+(Low8bit-0xa0-1)), 0);
    //lseek(fd, 32*((High8bit-0xa1)*95+(Low8bit-0xa1)), 0);//GB2312
    if(Low8bit < 0x7f)
    {
        lseek(fd, 32*((High8bit-0x81)*190+(Low8bit-0x40)), 0);//GBK
    }
    else {
        lseek(fd, 32*((High8bit-0x81)*190+(Low8bit-0x41)), 0);//GBK
    }

    //lseek(&fsrc,32*((High8bit-0xa0-1)*94+(Low8bit-0xa0-1)));
    read(fd, buffer, 32);

    close(fd);
    return 0;
}

//void PutChinese(uint16_t x, uint16_t y, uint8_t *c, uint32_t PenColor, uint32_t BackColor)
//{
//    uint16_t i=0;
//    uint16_t j=0;
//
//    uint8_t buffer[32];
//    uint16_t tmp_char=0;
//
//    if(!Find_Chinese(c,buffer))
//    {
////        LCD_WindowMax(x,y,x+16,y+16);
////        LCD_SetCursor(x,y);
////        LCD_WriteRAM_Prepare();
//        LCD_Address_Set(x,y,x+16,y+16);
//        for (i=0;i<16;i++)
//        {
//            tmp_char=buffer[i*2];
//            tmp_char=(tmp_char<<8);
//            tmp_char|=buffer[2*i+1];
//
//            for (j=0;j<16;j++)
//            {
//                if((tmp_char<<j) & 0x8000)
//                    //LCD_WriteRAM(PenColor);
//                    LCD_WR_DATA(PenColor);
//                else
//                    //LCD_WriteRAM(BackColor);
//                    LCD_WR_DATA(BackColor);
//            }
//        }
////        LCD_WindowMax(0x00,0x00,239,319);   /*鎭㈠绐椾綋澶у皬*/
//    }
//}

void LCD_ShowChinese16x16(uint16_t x,uint16_t y,uint8_t *s,uint16_t fc,uint16_t bc,uint8_t sizey,uint8_t mode)
{
    uint8_t i,j,m=0;
    uint16_t k;
    uint16_t HZnum;
    uint16_t TypefaceNum;
    uint16_t x0=x;

    uint8_t buffer[32];

    //rt_kprintf("High = %2x,Low = %2x\r\n",s[0],s[1]);
    if(Find_Chinese(s,buffer))
    {
        return;
    }
//    for(i = 0; i < 31; i++)
//    {
//        rt_kprintf("%2x,",buffer[i]);
//    }
//    rt_kprintf("%2x\r\n",buffer[i]);


    TypefaceNum=(sizey/8+((sizey%8)?1:0))*sizey;
    //HZnum=sizeof(tfont16)/sizeof(typFNT_GB16);

    LCD_Address_Set(x,y,x+sizey-1,y+sizey-1);
    for(i=0;i<TypefaceNum;i++)
    {
        for(j=0;j<8;j++)
        {
            if(!mode)
            {
                if(buffer[i]&(0x01<<j))LCD_WR_DATA(fc);
                else LCD_WR_DATA(bc);
                m++;
                if(m%sizey==0)
                {
                    m=0;
                    break;
                }
            }
            else//鍙犲姞鏂瑰紡
            {
                if(buffer[i]&(0x01<<j)) LCD_DrawPoint(x,y,fc);//鐢讳竴涓偣
                x++;
                if((x-x0)==sizey)
                {
                    x=x0;
                    y++;
                    break;
                }
            }
        }
    }
}


/******************************************************************************
      ˵ʾִ
      ݣx,yʾ
                *s Ҫʾĺִ
                fc ֵɫ
                bc ֵıɫ
                sizey ֺ ѡ 16 24 32
                mode:  0ǵģʽ  1ģʽ
      ֵ  
******************************************************************************/
void LCD_ShowChinese(uint16_t x,uint16_t y,uint8_t *s,uint16_t fc,uint16_t bc,uint8_t sizey,uint8_t mode)
{
    while(*s!=0)
    {
        if(sizey==16)
        {
            LCD_ShowChinese16x16(x,y,s,fc,bc,sizey,mode);
        }
        else {
            return ;
        }

//        if(sizey==12) //LCD_ShowChinese12x12(x,y,s,fc,bc,sizey,mode);
//        else if(sizey==16) LCD_ShowChinese16x16(x,y,s,fc,bc,sizey,mode);
//        else if(sizey==24) //LCD_ShowChinese24x24(x,y,s,fc,bc,sizey,mode);
//        else if(sizey==32) //LCD_ShowChinese32x32(x,y,s,fc,bc,sizey,mode);
//        else return;
        s+=2;
        x+=sizey;
    }
}


/*********************************************************************************
*     ƣvoid LCD_Put_16(uint16_t x, uint16_t y, uint8_t *str,uint32_t PenColor, uint32_t BackColor)
* ڲuint16_t x,uint16_t y ַλ
                        uint8_t  *c      ֵָ
                        uint32_t PenColorɫ
                        uint32_t BackColorɫ
* ڲʵʾַ
*     ܣ㺺ʾ
* ˵    
*********************************************************************************/
uint16_t LCD_Put_16(uint16_t x, uint16_t y, uint8_t *str,uint32_t PenColor, uint32_t BackColor)
{
    uint16_t num = 0;
    while(*str != 0)
    {
        if (x>MAX_HZ_POSX)  /*֤ʣĿռܷһ16*16֣ */
        {
            y+=16;
            x=0;
        }
        if (y>MAX_HZ_POSY)  /*֤ʣĿռܷһ16*16֣ */
        {
            break;
        }

        if ( *str < 0x80) /* ʾֱַַӴfont.hãСΪ16*8 */
        {
            if(*str == '\r')
            {
                //y+=16;
                x=0;
                num++;
            }
            else if(*str == '\n')
            {
                y+=16;
                x = 0;
                num++;
                if (y>MAX_HZ_POSY)  /*֤ʣĿռܷһ16*16֣ */
                {
                    break;
                }
            }
            else {
                LCD_ShowChar(x,y,*str,PenColor,BackColor,16,0);
                x+= 8;
                num++;
            }
            str++;
        }
        else                    /*ڵ0x80жΪַ*/
        {
            LCD_ShowChinese16x16(x,y,str,PenColor,BackColor,16,0);//ʾһ
            str+=2;
            x+=16;
            num+=2;
        }

    }
    return num;
}
