/*
 * Copyright (c) 2006-2021, RT-Thread Development Team
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Change Logs:
 * Date           Author       Notes
 * 2021-10-31     Administrator       the first version
 */
#include <rtthread.h>
#include "board.h"
#include "lcd/lcd_init.h"

uint8_t lcd_clk;               //PE6
uint8_t lcd_mosi;              //PE1
uint8_t lcd_reset;             //PA1
uint8_t lcd_rs;                //PE7
uint8_t lcd_cs;                //PE0
uint8_t lcd_miso;              //PA0

uint8_t lcd_cs2;               //PE3
uint8_t lcd_pen;               //PE2

void LCD_GPIO_Init(void)
{
    //lcdŶ
    lcd_clk   = rt_pin_get("PE.6");
    lcd_mosi  = rt_pin_get("PE.1");
    lcd_reset = rt_pin_get("PA.1");
    lcd_rs    = rt_pin_get("PE.7");
    lcd_cs    = rt_pin_get("PE.0");
    lcd_miso  = rt_pin_get("PA.0");

    //Ŷ
    lcd_cs2   = rt_pin_get("PE.3");
    lcd_pen   = rt_pin_get("PE.2");

    rt_pin_mode(lcd_clk, PIN_MODE_OUTPUT);
    rt_pin_mode(lcd_mosi, PIN_MODE_OUTPUT);
    rt_pin_mode(lcd_reset, PIN_MODE_OUTPUT);
    rt_pin_mode(lcd_rs, PIN_MODE_OUTPUT);
    rt_pin_mode(lcd_cs, PIN_MODE_OUTPUT);
    rt_pin_mode(lcd_miso, PIN_MODE_INPUT_PULLUP);

    rt_pin_mode(lcd_pen, PIN_MODE_INPUT_PULLDOWN);
    rt_pin_mode(lcd_cs2, PIN_MODE_OUTPUT);

    rt_pin_write(lcd_clk, PIN_HIGH);
    rt_pin_write(lcd_mosi, PIN_HIGH);
    rt_pin_write(lcd_reset, PIN_HIGH);
    rt_pin_write(lcd_rs, PIN_HIGH);
    rt_pin_write(lcd_cs, PIN_HIGH);
    rt_pin_write(lcd_cs2, PIN_HIGH);
}


void LCD_Writ_Bus(uint8_t dat)
{
    uint8_t i;
    LCD_CS_Clr();
    for(i=0;i<8;i++)
    {
        LCD_SCLK_Clr();
        if(dat&0x80)
        {
           LCD_MOSI_Set();
        }
        else
        {
           LCD_MOSI_Clr();
        }
        LCD_SCLK_Set();
        dat<<=1;
    }
  LCD_CS_Set();
}


void LCD_WR_DATA8(uint8_t dat)
{
    LCD_Writ_Bus(dat);
}



void LCD_WR_DATA(uint16_t dat)
{
    LCD_Writ_Bus(dat>>8);
    LCD_Writ_Bus(dat);
}



void LCD_WR_REG(uint8_t dat)
{
    LCD_DC_Clr();
    LCD_Writ_Bus(dat);
    LCD_DC_Set();
}



void LCD_Address_Set(uint16_t x1,uint16_t y1,uint16_t x2,uint16_t y2)
{
        LCD_WR_REG(0x2a);
        LCD_WR_DATA(x1);
        LCD_WR_DATA(x2);
        LCD_WR_REG(0x2b);
        LCD_WR_DATA(y1);
        LCD_WR_DATA(y2);
        LCD_WR_REG(0x2c);
}

void LCD_Init(void)
{
    LCD_GPIO_Init();//初始化GPIO

    LCD_RES_Clr();//复位
    rt_thread_mdelay(100);
    LCD_RES_Set();
    rt_thread_mdelay(100);

    LCD_BLK_Set();//打开背光
    rt_thread_mdelay(100);

    //************* Start Initial Sequence **********//
    LCD_WR_REG(0x11); //Sleep out
    rt_thread_mdelay(120);              //Delay 120ms
    //************* Start Initial Sequence **********//
    LCD_WR_REG(0xCF);
    LCD_WR_DATA8(0x00);
    LCD_WR_DATA8(0xD9);
    LCD_WR_DATA8(0X30);

    LCD_WR_REG(0xED);
    LCD_WR_DATA8(0x64);
    LCD_WR_DATA8(0x03);
    LCD_WR_DATA8(0X12);
    LCD_WR_DATA8(0X81);

    LCD_WR_REG(0xE8);
    LCD_WR_DATA8(0x85);
    LCD_WR_DATA8(0x10);
    LCD_WR_DATA8(0x78);

    LCD_WR_REG(0xCB);
    LCD_WR_DATA8(0x39);
    LCD_WR_DATA8(0x2C);
    LCD_WR_DATA8(0x00);
    LCD_WR_DATA8(0x34);
    LCD_WR_DATA8(0x02);

    LCD_WR_REG(0xF7);
    LCD_WR_DATA8(0x20);

    LCD_WR_REG(0xEA);
    LCD_WR_DATA8(0x00);
    LCD_WR_DATA8(0x00);

    LCD_WR_REG(0xC0);    //Power control
    LCD_WR_DATA8(0x21);   //VRH[5:0]

    LCD_WR_REG(0xC1);    //Power control
    LCD_WR_DATA8(0x12);   //SAP[2:0];BT[3:0]

    LCD_WR_REG(0xC5);    //VCM control
    LCD_WR_DATA8(0x32);
    LCD_WR_DATA8(0x3C);

    LCD_WR_REG(0xC7);    //VCM control2
    LCD_WR_DATA8(0XC1);

    LCD_WR_REG(0x36);    // Memory Access Control
    if(USE_HORIZONTAL==0)LCD_WR_DATA8(0x08);
    else if(USE_HORIZONTAL==1)LCD_WR_DATA8(0xC8);
    else if(USE_HORIZONTAL==2)LCD_WR_DATA8(0x78);
    else LCD_WR_DATA8(0xA8);

    LCD_WR_REG(0x3A);
    LCD_WR_DATA8(0x55);

    LCD_WR_REG(0xB1);
    LCD_WR_DATA8(0x00);
    LCD_WR_DATA8(0x18);

    LCD_WR_REG(0xB6);    // Display Function Control
    LCD_WR_DATA8(0x0A);
    LCD_WR_DATA8(0xA2);



    LCD_WR_REG(0xF2);    // 3Gamma Function Disable
    LCD_WR_DATA8(0x00);

    LCD_WR_REG(0x26);    //Gamma curve selected
    LCD_WR_DATA8(0x01);

    LCD_WR_REG(0xE0);    //Set Gamma
    LCD_WR_DATA8(0x0F);
    LCD_WR_DATA8(0x20);
    LCD_WR_DATA8(0x1E);
    LCD_WR_DATA8(0x09);
    LCD_WR_DATA8(0x12);
    LCD_WR_DATA8(0x0B);
    LCD_WR_DATA8(0x50);
    LCD_WR_DATA8(0XBA);
    LCD_WR_DATA8(0x44);
    LCD_WR_DATA8(0x09);
    LCD_WR_DATA8(0x14);
    LCD_WR_DATA8(0x05);
    LCD_WR_DATA8(0x23);
    LCD_WR_DATA8(0x21);
    LCD_WR_DATA8(0x00);

    LCD_WR_REG(0XE1);    //Set Gamma
    LCD_WR_DATA8(0x00);
    LCD_WR_DATA8(0x19);
    LCD_WR_DATA8(0x19);
    LCD_WR_DATA8(0x00);
    LCD_WR_DATA8(0x12);
    LCD_WR_DATA8(0x07);
    LCD_WR_DATA8(0x2D);
    LCD_WR_DATA8(0x28);
    LCD_WR_DATA8(0x3F);
    LCD_WR_DATA8(0x02);
    LCD_WR_DATA8(0x0A);
    LCD_WR_DATA8(0x08);
    LCD_WR_DATA8(0x25);
    LCD_WR_DATA8(0x2D);
    LCD_WR_DATA8(0x0F);
    LCD_WR_REG(0x29);    //Display on
}
